{-
     Author: Georgi Koshov
     E-mail: georgi@koshov.com
     Program: Maze Generator

     Description: This is a simple algorithm for generating a semi-three-dimensional labyrinth. It relies on turtle graphics and uses various techniques for layering an image which seems 3-dimensional to the viewer.

     How to use: Just type "runghc mazeGen.hs" in the terminal and it will execute. The program requires the raphics.UI.GLUT and Graphics.Rendering.OpenGL libraries to be installed.
     You can change the values in the settings section, but keep in mind that this method has certain limitations.

     Feedback: If you have any comments or ideas related to this program, feel free to contact me on georgi@koshov.com, any feedback is welcome!

-}



import LSystem
import Test.QuickCheck
import Fractals

main :: IO()
main = display (signature :#: base :#: maze)

-- Settings
angle :: Angle
angle = 15

perspective :: Float
perspective = 0.6

height :: Int
height = 50

--    -Chose between shadeR/shadeB/shadeG or build yours!
shade :: Int -> Pen
shade i = shadeR i

-- Maze Drawing

maze = stackCommand (Turn angle :#: (shorten perspective (hilbert 4))) height

--    -Create layers of shifted images, that build up the walls 
stackCommand :: Command -> Int -> Command
stackCommand c i = foldr (:#:) Sit 
                 [Branch (GrabPen (shade (i-x)) :#: Go ((fromIntegral x)/5) :#: c) | x <- [0..i]]

--    -Decrese the height of the maze (all vertical segments) to create perspective
shorten :: Float -> Command -> Command
shorten f c = join (shortRec (split c))
       where
         shortRec [Sit] = []
         shortRec ((Turn x):(Go y):(Turn z):(Go w):xs) = 
                  (Turn x):(Go y):(Turn z):(Go (w*f)):shortRec xs
         shortRec ((Go x):(Turn y):(Go z):(Turn w):xs) = 
                  (Go (x*f)):(Turn y):(Go z):(Turn w):shortRec xs
         shortRec ((Turn x):(Go y):xs) = 
                  (Turn x):(Go y):shortRec xs
         shortRec ((Go x):(Turn y):xs) = 
                  (Go (x*f)):(Turn y):shortRec xs
         shortRec ((GrabPen x): xs) = shortRec xs
         shortRec x = x

--    -Custom gradient color
shadeR, shadeG, shadeB :: Int -> Pen
shadeR x = Colour (0.5+(fromIntegral x)*0.01) (1-(fromIntegral x)*0.01) 0.1
shadeG x = Colour (0+(fromIntegral x)*0.01) (0.9-(fromIntegral x)*0.01) (0.1+(fromIntegral x)*0.01)
shadeB x = Colour 0.1 (0.5+(fromIntegral x)*0.01) (1-(fromIntegral x)*0.01)


--Base bellow the maze
--    -Draw the base diagonally outside the maze, since its width and height are larger
base :: Command
base = Branch (offSide :#: Turn angle :#: GrabPen gray :#: ( shorten perspective (baseForm 160 8000)))
    where offSide = Turn (135) :#: Go 5 :#: Turn (-135)
--    -Draw a square spiral with big density
--        -May not be as quick as drawing an OpenGL square, but is made by the turtle
baseForm :: Distance -> Int -> Command
baseForm side n  |  side <= 0 || n == 0  =  Sit 
                 |  otherwise   =  Go side :#: Turn (-90) :#: (baseForm (side-0.02) (n-1) )

-- Signature
--    -Every masterpiece should be signed by its painter...
--        -Try debugging that!
signature :: Command
signature = Branch (Turn (-90) :#: Go 30 :#: GrabPen green :#: Go 80 :#: Turn (-90) :#: Go 4 :#: Turn (90) :#: Go 2 :#: Turn (90) :#: Go 8 :#: Turn (-90) :#: Go 2 :#: Turn (-90) :#: Go 6 :#: Turn (90) :#: Go 2 :#: Turn (90) :#: Go 2 :#: Turn (-90) :#: Go 2 :#: Turn (-90) :#: Go 6 :#: Turn (90) :#: Go 2 :#: Turn (90) :#: Go 12 :#: Turn (-90) :#: Go 2 :#: Turn (-90) :#: Go 6 :#: Turn (90) :#: Go 2 :#: Turn (-90) :#: Go 4 :#: Turn (90) :#: Go 2 :#: Turn (90) :#: Go 4 :#: Turn (-90) :#: Go 20   :#:   GrabPen Inkless :#: Turn 180 :#: Go 8 :#: Turn (-90) :#: Go 3 :#: GrabPen green :#: Turn 160 :#: Go 2 :#: Turn (-140) :#: Go 2 :#: Turn 70 :#: GrabPen Inkless :#:  Go 2 :#: GrabPen green :#: Turn 90 :#: Go 2 :#: Turn 90 :#: Go 1 :#: Turn 90 :#: Go 2 :#: Turn 90 :#: Go 1 :#: GrabPen Inkless :#: Go 1 :#: GrabPen green :#: Turn 90 :#: Go 2 :#: Turn 180 :#: Go 1 :#: Turn 90 :#: Go 1 :#: Turn 90 :#: Go 1 :#: Turn (-180) :#: Go 2 :#: Turn 90 :#: GrabPen Inkless :#: Go 1 :#: GrabPen green :#: Go 1 :#: Turn 90 :#: Go 1 :#: Turn 90 :#: Go 1 :#: Turn (-90) :#: Go 1 :#: Turn (-90) :#: Go 1 :#: GrabPen Inkless :#: Go 2 :#: Turn (-90) :#: GrabPen green :#: Go 2 :#: Turn (-90) :#: Go 1 :#: Turn (-90) :#: Go 2 :#: Turn (-90) :#: Go 1 :#: GrabPen Inkless :#: Go 1 :#: Turn (-45) :#: GrabPen green :#: Go 2 :#: Turn (-90) :#: Go 2 :#: Turn 180 :#: Go 2 :#: Turn (-135) :#: Go 1.5 :#: Turn (-180) :#: Go 3 )

